﻿namespace BPA.MES.Base.Application.Services.Template
{
    /// <summary>
    /// 名    称 ：  
    /// 创 建 人 ：yangxiao  
    /// 创建时间 ：  2023/7/31 18:03:25 
    /// 描    述 ：  
    /// </summary>
    [ApiDescriptionSettings("我是分组名称-模板", Name = "Template", Tag = "我的名字叫模板", KeepName = true, SplitCamelCase = true, KeepVerb = true)]
    public class TemplateService : ITemplateService, ITransient, IDynamicApiController
    {
        private readonly ISqlSugarClient _dbContext;
        public TemplateService(ISqlSugarClient db)
        {
            _dbContext = db;
        }
        /// <summary>
        /// 增加
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<bool> Add(TemplateAddInput input)
        {
            var r_entity = await _dbContext.Queryable<TemplateEntity>().FirstAsync(x => x.Name == input.Name);
            if (r_entity != null)
            {
                throw Oops.Bah("名称已存在!");
            }
            TemplateEntity entity = input.Adapt<TemplateEntity>();
            return await _dbContext.Insertable(entity).IgnoreColumns(ignoreNullColumn: true).ExecuteCommandIdentityIntoEntityAsync();
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost]
        public async Task<bool> Del(TemplateDelInput input)
        {
            var res = await _dbContext.Deleteable<TemplateEntity>().Where(x => x.Id == input.Id).ExecuteCommandHasChangeAsync();
            return res;
        }
        /// <summary>
        /// 详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpGet]
        public async Task<TemplateOutput> Detail(string Id)
        {
            var entity = await _dbContext.Queryable<TemplateEntity>().FirstAsync(x => x.Id == Id);
            TemplateOutput output = entity.Adapt<TemplateOutput>();
            return output;
        }
        /// <summary>
        /// 列表
        /// </summary>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost]
        public async Task<List<TemplateOutput>> List(TemplateQueryInput input)
        {
            var entity = await _dbContext.Queryable<TemplateEntity>().ToListAsync();
            List<TemplateOutput> output = entity.Adapt<List<TemplateOutput>>();
            return output;
        }
        /// <summary>
        /// 分页
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<SqlSugarPagedList<TemplateOutput>> PagedList(TemplateQueryPageInput input)
        {
            var entity = await _dbContext.Queryable<TemplateEntity>()
                .WhereIF(!string.IsNullOrEmpty(input.Name), x => x.Name.Contains(input.Name))
                .WhereIF(!string.IsNullOrEmpty(input.Code), x => x.Code.Contains(input.Code))
                .ToPagedListAsync(input.PageIndex, input.PageSize);
            SqlSugarPagedList<TemplateOutput> output = entity.Adapt<SqlSugarPagedList<TemplateOutput>>();
            return output;
        }
        /// <summary>
        /// 下拉
        /// </summary>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpGet]
        public async Task<dynamic> SelectList()
        {
            var entity = await _dbContext.Queryable<TemplateEntity>().Select(x => new
            {
                key = x.Id,
                value = x.Name
            }).ToListAsync();
            return entity;
        }
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost]
        public async Task<bool> Update(TemplateUpdateInput input)
        {
            var entity = await _dbContext.Queryable<TemplateEntity>().Where(x => x.Id == input.Id).FirstAsync();
            entity = input.Adapt<TemplateEntity>();
            var res = await _dbContext.Updateable(entity).IgnoreColumns(true).ExecuteCommandHasChangeAsync();
            return res;
        }
    }
}
